from typing import List, Optional, Dict, Any
from pydantic import Field, model_validator, field_validator, ConfigDict
from abc import abstractmethod

from ...utils.types.source_types import SourceType, SourceCheckerType
from ...utils.custom_exceptions import InvalidSourceChecker
from ...utils.types.identifier import StrObjectId
from .helpers import SourceHelpers
from ...base_models import CompanyAssetModel
from ...utils.types.serializer_type import SerializerType

import logging
logger = logging.getLogger("source_base")

class SourceBase(CompanyAssetModel):
    agent_email: Optional[str] = Field(default=None)
    description: Optional[str] = Field(default="")
    tags: List[StrObjectId] = Field(default_factory=list)
    products: List[StrObjectId] = Field(default_factory=list)
    flow: List[StrObjectId] = Field(default_factory=list)
    trackeable: bool = Field(default=True)
    category: str = Field(default="")
    source_checker: SourceCheckerType
    name: str = Field()

    model_config = ConfigDict(extra='ignore', exclude_none=True) #type: ignore

    def model_dump(self, *args, **kwargs) -> Dict:
        kwargs["by_alias"] = True
        data = super().model_dump(*args, **kwargs)
        data["type"] = self.type
        return data

    def model_dump_json(self, *args, **kwargs) -> Dict[str, Any]:
        kwargs["by_alias"] = True
        data = super().model_dump_json(*args, **kwargs)
        data["type"] = self.type
        return data

    @field_validator('source_checker', mode="before")
    @classmethod
    def lowercase_source_checker(cls, v: str) -> str:
        return v.lower()

    @model_validator(mode='after')
    def validate_source(self):
        if not SourceHelpers.is_valid_source_checker(source_type=self.type,source_checker=self.source_checker):
            raise InvalidSourceChecker(f"Source checker {self.source_checker} not valid for source type {self.type} | Allowed ones are {SourceHelpers.get_source_checkers(self.type)}")
        if self.category == "":
            self.category = self.type
        return self

    @model_validator(mode='before')
    def validate_lists(cls, data: Dict) -> Dict:
        if "products" in data and data["products"] is None:
            data["products"] = []
        if "flow" in data and data["flow"] is None:
            data["flow"] = []
        if "tags" in data and data["tags"] is None:
            data["tags"] = []
        return data

    @property
    @abstractmethod
    def type(self) -> SourceType:
        pass